//
//  BFVideoThumbProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/3.
//

import AVFoundation
import BFCommonKit
import BFUIKit
import Foundation
import SnapKit
import UIKit

class BFVideoThumbProgressView: UIView {
    var recordItem: BFRecordItemModel? {
        didSet {
            if recordItem?.mediaType == .VIDEO {
                dealWithVideoThub()
            } else if recordItem?.mediaType == .IMAGE {
                dealWithImageThub()
            }
        }
    }

    var dragScrollProgressHandle: ((Bool, Float) -> Void)?
    var dragEndHandle: ((Float) -> Void)?
    var dragStartHandle: (() -> Void)?
    var isDrag = false

    let thumbImageWidth = 70.0

    let fetchThumbStrategy: BFVideoThumbProgressStrategyProtocol = BFVideoThumbProgressStrategy()

    var progress: Double = 0 {
        didSet {
            if let second = recordItem?.videoAsset?.duration.seconds, second > 0 {
                let w = progressView.contentSize.width - width
                progressView.contentOffset = CGPoint(x: progress * Double(w) / second, y: 0)
            }
        }
    }

    var thumbImgs = [UIImage]()

    lazy var progressView: BFAutolayoutScrollView = {
        let sv = BFAutolayoutScrollView()
        sv.bounces = false
//        sv.backgroundColor = .clear
//        sv.backgroundColor = UIColor.hexColor(hexadecimal: "#888888",alpha:0.3)
        sv.backgroundColor = UIColor(red: 0, green: 0, blue: 0, alpha: 0.3)
        sv.decelerationRate = .fast
        sv.showsHorizontalScrollIndicator = false
        sv.delegate = self
        return sv
    }()

    lazy var progessIndicateBackV: UIView = {
        let vv = UIView()
        return vv
    }()

    // MARK: - 生命周期

    override init(frame: CGRect) {
        super.init(frame: frame)
        addSubview(progressView)

        let line = UIView()
        line.backgroundColor = .white
        line.layer.borderColor = UIColor.black.cgColor
        line.layer.borderWidth = 0.3
        line.layer.cornerRadius = 1.5
        addSubview(line)
        line.snp.makeConstraints { make in
            make.width.equalTo(3)
            make.center.height.equalToSuperview()
        }
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    override func layoutSubviews() {
        super.layoutSubviews()

        progressView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
    }

    func dealWithVideoThub() {
        guard let videoAsset = recordItem?.videoAsset else {
            return
        }
        let dur = videoAsset.duration.seconds
        if dur > 0 {
            let fps = Double(fetchThumbStrategy.frameNumberOfVideo(assetDuration: dur)) / dur
            splitVideoFileUrlFps(urlAsset: videoAsset, fps: fps) { [weak self] images in
                self?.addThubImages(images: images)
            }
        }
    }

    func dealWithImageThub() {
        guard let image = recordItem?.coverImg else {
            return
        }
        addThubImages(images: Array(repeating: image, count: 3))
    }

    func addThubImages(images: [UIImage]) {
        if images.count > 0 {
            thumbImgs = images
            DispatchQueue.main.async { [weak self] in
                self?.progressView.contentView.subviews.forEach { subview in
                    subview.removeFromSuperview()
                }
                if let sself = self {
                    var lastiv = UIImageView()
                    for (i, img) in images.enumerated() {
                        let iv = UIImageView(image: img)
                        iv.contentMode = .scaleAspectFill
                        iv.clipsToBounds = true
                        sself.progressView.contentView.addSubview(iv)
                        iv.snp.makeConstraints { make in
                            make.left.equalTo(CGFloat(i) * CGFloat(sself.thumbImageWidth) + sself.width * 0.5)
                            make.top.bottom.equalToSuperview()
                            make.height.equalTo(50)
                            make.width.equalTo(sself.thumbImageWidth)
                        }
                        lastiv = iv
                    }
                    lastiv.snp.makeConstraints { make in
                        make.right.equalTo(sself.width * -0.5)
                    }

                    sself.progressView.contentView.addSubview(sself.progessIndicateBackV)
                    sself.progessIndicateBackV.snp.makeConstraints { make in
                        make.left.equalTo(sself.width * 0.5)
                        make.right.equalTo(sself.width * -0.5)
                        make.bottom.equalToSuperview()
                        make.height.equalTo(6)
                    }
                }
            }
        }
    }
}

extension BFVideoThumbProgressView: UIScrollViewDelegate {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if isDrag {
            let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - width)
            dragScrollProgressHandle?(false, Float(dur))
        }
    }

    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - width)
        dragStartHandle?()
        dragScrollProgressHandle?(true, Float(dur))
    }

    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - width)
            isDrag = false
            dragEndHandle?(Float(dur))
        }
    }

    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - width)
        isDrag = false
        dragEndHandle?(Float(dur))
    }
}
